<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="keditbookmarks" version="26">

<MenuBar>

<Menu name="file" noMerge="1"><text>&amp;File</text>
  <Action name="file_open"/>
  <Action name="file_save_as"/>
  <Separator/>
  <Action name="file_quit"/>
</Menu>

<Menu name="edit"><text>&amp;Edit</text>
  <!-- undo/redo/cut/copy/paste stuff is
       merged (?) at this point (ui_standards.rc) -->
  <!-- Various things -->
  <Action name="rename" append="edit_paste_merge"/>
  <Action name="delete" append="edit_paste_merge"/>
  <Separator/>
  <Action name="changeurl"/>
  <Action name="changecomment"/>
  <Action name="changeicon"/>
</Menu>

<Menu name="view"><text>&amp;View</text>
  <Action name="expandall"/>
  <Action name="collapseall"/>
</Menu>

<Menu name="folder"><text>&amp;Folder</text>
  <Action name="newfolder"/>
  <Action name="insertseparator"/>
  <Separator/>
  <Action name="sort"/>
  <Action name="recursivesort"/>
  </Menu>

<Menu name="bookmark"><text>&amp;Bookmark</text>
  <Action name="newbookmark"/>
</Menu>

<Menu name="settings"><text>&amp;Settings</text>
<!--  <Action name="settings_splitview" append="save_merge"/> -->
</Menu>

</MenuBar>

<ToolBar name="mainToolBar" noMerge="1" fullWidth="true"><text>Main Toolbar</text>
  <Action name="edit_undo"/>
  <Action name="edit_redo"/>
  <Separator/>
  <Action name="newbookmark"/>
  <Action name="newfolder"/>
</ToolBar>

<Menu name="popup_folder">
  <!-- Stuff for folders -->
  <Action name="setastoolbar"/>
  <Action name="sort"/>
  <Action name="recursivesort"/>
   <Separator/>
  <Action name="testlink"/>
  <Separator/><!-- Clipboard -->
  <Action name="edit_undo"/>
  <Action name="edit_redo"/>
  <Action name="edit_cut"/>
  <Action name="edit_copy"/>
  <Action name="edit_paste"/>
  <Separator/><!-- Edition -->
  <Action name="rename"/>
  <Action name="changeurl"/>
  <Action name="changeicon"/>
  <Separator/><!-- Creation -->
  <Action name="newbookmark"/>
  <Action name="newfolder"/>
  <Action name="insertseparator"/>
  <Separator/><!-- Dangerous actions :/ -->
  <Action name="delete"/>
</Menu>

<Menu name="popup_bookmark">
  <!-- Stuff for bookmarks -->
  <Separator/><!-- Clipboard stuff -->
  <Action name="edit_undo"/>
  <Action name="edit_redo"/>
  <Action name="edit_cut"/>
  <Action name="edit_copy"/>
  <Action name="edit_paste"/>
  <Separator/><!-- Edition -->
  <Action name="rename"/>
  <Action name="changeurl"/>
  <Action name="changeicon"/>
  <Separator/><!-- Creation -->
  <Action name="newbookmark"/>
  <Action name="newfolder"/>
  <Action name="insertseparator"/>
  <Separator/><!-- Dangerous actions :/ -->
  <Action name="delete"/>
</Menu>

<!--         APPLICATION STATES        -->

<State name="notreadonly">
  <Enable>
    <Action name="settings_saveonclose"/>
  </Enable>
</State>

<State name="normal">
  <Enable>
    <Action name="file_open"/>
    <Action name="file_save_as"/>
    <Action name="file_quit"/>
    <Action name="expandall"/>
    <Action name="collapseall"/>
  </Enable>
</State>

<State name="disablestuff">
  <Disable>
    <Action name="changecomment"/>
    <Action name="changeicon"/>
    <Action name="changeurl"/>
    <Action name="delete"/>
    <Action name="edit_copy"/>
    <Action name="edit_cut"/>
    <Action name="edit_paste"/>
    <Action name="edit_redo"/>
    <Action name="edit_undo"/>
    <Action name="insertseparator"/>
    <Action name="newbookmark"/>
    <Action name="newfolder"/>
    <Action name="nexthit"/>
    <Action name="openlink"/>
    <Action name="rename"/>
    <Action name="setastoolbar"/>
    <Action name="sort"/>
    <Action name="recursivesort"/>
    <Action name="settings_saveonclose"/>
  </Disable>
</State>

</gui>
